"""Add missing permissions for `tester` and `annotator` roles.

Reason:
Both `tester` and `annotator` cannot have conversations because
the currently active model needs to be fetched and both roles
don't have a permission to do that.

Revision ID: b67a67032c7f
Revises: 8893bba2a522

"""
import logging
import sqlalchemy as sa
from alembic import op
from sqlalchemy.exc import IntegrityError
import rasax.community.database.schema_migrations.alembic.utils as migration_utils
from rasax.community.services.user_service import TESTER, ANNOTATOR


# revision identifiers, used by Alembic.
revision = "b67a67032c7f"
down_revision = "8893bba2a522"
branch_labels = None
depends_on = None

TABLE_NAME = "conversation"
CREATED_BY_COLUMN = "created_by"
FOREIGN_KEY_NAME = "fk_conversation_username"
NEW_INDEX_NAME = "conversation_created_by_idx"

logger = logging.getLogger(__name__)


def upgrade():
    # `TESTER` and `ANNOTATOR` need to be able to fetch models (in order to have conversations)
    migration_utils.add_new_permission_to(ANNOTATOR, "models.view.*")
    migration_utils.add_new_permission_to(TESTER, "models.view.*")

    # Create `created_by` column
    if migration_utils.get_column(TABLE_NAME, CREATED_BY_COLUMN) is None:
        migration_utils.create_column(
            TABLE_NAME, sa.Column(CREATED_BY_COLUMN, sa.String(255), nullable=True)
        )

        with op.batch_alter_table(TABLE_NAME) as batch_op:
            batch_op.create_foreign_key(
                FOREIGN_KEY_NAME, "rasa_x_user", [CREATED_BY_COLUMN], ["username"]
            )

    # Index for `created_by` column
    if migration_utils.find_matching_index(NEW_INDEX_NAME, TABLE_NAME) is None:
        with op.batch_alter_table(TABLE_NAME) as batch_op:
            batch_op.create_index(NEW_INDEX_NAME, ["created_by"])


def downgrade():
    # Remove index for `created_by` column
    if migration_utils.find_matching_index(NEW_INDEX_NAME, TABLE_NAME) is not None:
        with op.batch_alter_table(TABLE_NAME) as batch_op:
            batch_op.drop_index(NEW_INDEX_NAME)

    # Remove `created_by` column
    if migration_utils.get_column(TABLE_NAME, CREATED_BY_COLUMN) is not None:
        with op.batch_alter_table(TABLE_NAME) as batch_op:
            batch_op.drop_constraint(FOREIGN_KEY_NAME)

        migration_utils.drop_column(TABLE_NAME, CREATED_BY_COLUMN)

    # Remove additional roles for `TESTER` and `ANNOTATOR`
    migration_utils.delete_permission_from(ANNOTATOR, "models.view.*")
    migration_utils.delete_permission_from(TESTER, "models.view.*")
